@@ -28,7 +28,7 @@ pub(crate) fn evaluate_to_value<'a, I: Interrupt>(
     for _ in 0..missing_open_parens {
         tokens.insert(0, lexer::Token::Symbol(lexer::Symbol::OpenParens));
     }
-    let parsed = parser::parse_tokens(&tokens).map_err(|e| e.to_string())?;
+    let parsed = parser::parse_tokens(&tokens)?;
     let result = ast::evaluate(parsed, scope, context, int)?;
     Ok(result)
 }
